﻿using System;
using System.Windows;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Windows.Input;
using WpfGoogleMapClient.Commands;

namespace WpfGoogleMapClient
{
	/// <summary>
	/// MainWindow.xaml の相互作用ロジック
	/// </summary>
	public partial class MainWindow : Window
	{
		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		public MainWindow()
		{
			string uri       = String.Format( "file://{0}res\\index.html", AppDomain.CurrentDomain.BaseDirectory );
			this.DataContext = new MainWindowViewModel( uri, OnMoveMap, OnMoveMapToMarker, OnMarker ) { Address = "新宿" };

			this.InitializeComponent();
		}

		/// <summary>
		/// 移動コマンドが実行された時に呼び出されます。
		/// </summary>
		/// <param name="address">移動先となる住所。</param>
		private void OnMoveMap( string address )
		{
			this._webBrowser.InvokeScript( "moveMap", address );
		}

		/// <summary>
		/// マーカーの位置へマップを移動させるコマンドが実行された時に呼び出されます。
		/// </summary>
		private void OnMoveMapToMarker()
		{
			this._webBrowser.InvokeScript( "moveMapToMarker" );
		}

		/// <summary>
		/// マーカーの追加・削除を行うコマンドが実行された時に呼び出されます。
		/// </summary>
		/// <param name="isAdd">マーカーを追加する場合は true。削除ならば false。</param>
		private void OnMarker( bool isAdd )
		{
			this._webBrowser.InvokeScript( isAdd ? "addMarker" : "removeMarker" );
		}
	}

	/// <summary>
	/// MainWindow の Model と View を仲介するクラスです。
	/// </summary>
	class MainWindowViewModel
	{
		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		/// <param name="uri">ブラウザに表示するページの URI。</param>
		/// <param name="moveMap">移動コマンドが実行された時に呼び出されるコールバック メソッド。</param>
		/// <param name="moveMapToMarker">マーカーの位置へマップを移動させるコマンドが実行された時に呼び出されるコールバック メソッド。</param>
		/// <param name="marker">マーカーの追加・削除を行うコマンドが実行された時に呼び出されるコールバック メソッド。</param>
		public MainWindowViewModel( string uri, Action<string> moveMap, Action moveMapToMarker, Action< bool > marker )
		{
			this.Uri                    = uri;
			this.MapHost                = new MapHost();
			this.MoveMapCommand         = new DelegateCommand( () => { moveMap( this.Address ); } );
			this.MoveMapToMarkerCommand = new DelegateCommand( () => { moveMapToMarker(); } );
			this.MarkerCommand          = new DelegateCommand( () => { marker( !this.MapHost.IsExistMarker ); } );
		}

		/// <summary>
		/// マーカーの追加・削除を行うコマンドを取得します。
		/// </summary>
		public ICommand MarkerCommand { get; private set; }

		/// <summary>
		/// 現在設定されている Address の位置にマップを移動させるコマンドを取得します。
		/// </summary>
		public ICommand MoveMapCommand { get; private set; }

		/// <summary>
		/// マーカーの位置へマップを移動させるコマンドを取得します。
		/// </summary>
		public ICommand MoveMapToMarkerCommand { get; private set; }

		/// <summary>
		///  マップの操作を行う JavaScript に関連付けられるオブジェクトを取得します。
		/// </summary>
		public MapHost MapHost { get; private set; }

		/// <summary>
		/// マップに指定する住所を取得または設定します。
		/// </summary>
		public string Address { get; set; }

		/// <summary>
		/// ブラウザに表示するページの URI を取得します。
		/// </summary>
		public string Uri { get; private set; }
	}

	/// <summary>
	/// Google Map の操作を行う JavaScript に関連付けられるクラスです。
	/// </summary>
	[ComVisible( true )]
	public class MapHost : INotifyPropertyChanged
	{
		/// <summary>
		/// マップ上にマーカーが配置されている事を示す値を取得または設定します。
		/// </summary>
		public bool IsExistMarker
		{
			get
			{
				return this._isExistMarker;
			}
			set
			{
				this._isExistMarker = value;
				this.NotifyPropertyChanged( "IsExistMarker", "MarkerCommandText" );
			}
		}

		/// <summary>
		/// マップ上の緯度・経度を示す文字列を取得または設定します。
		/// </summary>
		public string Location
		{
			get
			{
				return this._location;
			}
			set
			{
				this._location = value;
				this.NotifyPropertyChanged( "Location" );
			}
		}

		/// <summary>
		/// マップ上のマーカーの緯度・経度を示す文字列を取得または設定します。
		/// </summary>
		public string MarkerLocation
		{
			get
			{
				return this._markerLocation;
			}
			set
			{
				this._markerLocation = value;
				this.NotifyPropertyChanged( "MarkerLocation" );
			}
		}

		/// <summary>
		/// マーカーへ実行可能な操作を示す文字列を取得します。
		/// </summary>
		public string MarkerCommandText
		{
			get { return this._isExistMarker ? "マーカー削除" : "マーカー追加"; }
		}

		/// <summary>
		/// マップ上にマーカーが配置されている事を示す値。
		/// </summary>
		private bool _isExistMarker;

		/// <summary>
		/// マップの中央移動座標。
		/// </summary>
		private string _location;

		/// <summary>
		/// マップ上のマーカー座標。
		/// </summary>
		private string _markerLocation;

		#region INotifyPropertyChanged メンバ

		/// <summary>
		/// プロパティが変更された事を通知するイベント ハンドラ。
		/// </summary>
		public event PropertyChangedEventHandler PropertyChanged;

		/// <summary>
		/// プロパティの変更を通知します。
		/// </summary>
		/// <param name="names">変更されたプロパティ名のコレクション。</param>
		protected void NotifyPropertyChanged( params string[] names )
		{
			if( this.PropertyChanged != null )
			{
				foreach( string name in names )
				{
					this.PropertyChanged( this, new PropertyChangedEventArgs( name ) );
				}
			}
		}

		#endregion
	}
}
